package bookExamples.ch26Graphics.draw2d.shapes;


import bookExamples.ch26Graphics.draw2d.*;
import gui.In;
import gui.run.RunButton;
import gui.run.RunSlider;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.beans.PropertyEditor;

public class Oval2d extends DJShape
        implements PropertyEditor, Intersects, Movable, Containment  {
    int x1 = 0;
    int y1 = 0;
    int h = 1;
    int w = 1;
    int xc = 0;
    int yc = 0;
    Vec2d center;


    public boolean supportsCustomEditor() {
        return true;
    }
    public boolean isPaintable() {
        return false;
    }

    public String getJavaInitializationString() {
         return null;
     }
    public String[] getTags() {
        return new String[0];
    }

    public void setAsText(String text) throws IllegalArgumentException {
    }

    public void paintValue(Graphics gfx, Rectangle box) {
    }
    public Object getValue() {
        return null;
    }

    public void setValue(Object value) {
    }


    public String getAsText() {
        return null;
    }


    public DJShape getInstance(int x1, int y1, int x2, int y2) {
        return new Oval2d(x1, y1, x2, y2);
    }

    public Point getCenter() {
        return new Point(xc, yc);
    }

    private void setWidth(int width){
        w=width;
        x1=xc-w/2;

    }

    private void setHeight(int height){
        h=height;
        y1=yc-h/2;
    }


    public Vec2d intersect(Ray2d ray) {

        double t1, t2,          /* where the ray intersects */
                // vDotV = distance square from sphere venterV2d to
                // ray origin
                vDotV,
                dDotV,
                dDotD,
                thc;        // length sqare of the half chord


        Vec2d v;     // vector to the venterV2d of the sphere from  ray origin
        Vec2d d;      // Direction vector

        /* use the closest approach algorithm */
        v = new Vec2d(-center.v[0], -center.v[1]);
        d = new Vec2d(ray.getDirection().v[0], ray.getDirection().v[1]);
        v.add(ray.getOrigin());
        int wOn2 = w / 2;
        int hOn2 = h / 2;
        v.v[0] = v.v[0] / wOn2;
        v.v[1] = v.v[1] / hOn2;
        d.v[0] = d.v[0] / wOn2;
        d.v[1] = d.v[1] / hOn2;

        // v = v - ray.p
        // loc = v^2
        vDotV = v.dot(v);
        dDotD = d.dot(d);

        // dDotV = D dot V =
        // cos(angle between ray and vec to venterV2d);
        dDotV = v.dot(d);   /* find the closest approach */

        /* compute the half chord square from the ray intersection to the
      intersection normal. */
        thc = (dDotV * dDotV) - dDotD * (vDotV - 1);

        if (thc < 0.0)
            return null;   /* ray misses the sphere */

        /* find the ray intersection */
        t1 = (-dDotV + Math.sqrt(thc)) / (dDotD);
        t2 = (-dDotV - Math.sqrt(thc)) / (dDotD);
        if ((t1 > 0) && (t2 > 0)) {
            if (t1 < t2)
                return ray.vecOnLine( t1);
            else
                return ray.vecOnLine( t2);
        } else if (t1 > 0)
            return ray.vecOnLine(t1);
        else if (t2 > 0)
            return ray.vecOnLine(t2);
        else
            return null;        // object behind
    }

    public Oval2d(int _x1, int _y1, int _x2, int _y2) {
        super(_x1, _y1, _x2, _y2);
        x1 = _x1;
        y1 = _y1;
        w = Math.abs(_x2 - x1);
        h = Math.abs(_y2 - y1);
        if (_x1 > _x2) x1 = _x2;
        if (_y1 > _y2) y1 = _y2;
        xc = x1 + w / 2;
        yc = y1 + h / 2;
        center =
                new Vec2d(xc, yc);
    }

    public void draw(Graphics g) {
         Graphics2D g2d = (Graphics2D) g;

        AffineTransform old = g2d.getTransform();
        AffineTransform affineTransform = getAffineTransform();
        g2d.transform(affineTransform);
        g2d.setColor(getForeground());
        g2d.drawOval(x1,y1, w, h);

        g2d.fillOval(xc,yc, 2, 2);
        g2d.drawString("(" + xc + "," + yc + ")", xc + 3, yc + 3);
        g2d.drawLine(xc, yc, xc, yc - h / 2);
        g2d.drawLine(xc, yc, xc - w / 2, yc);
        g2d.setTransform(old);
    }

    private Component setConstr(Component o, GridBagLayout l, int x, int y, int wx, int wy){
        GridBagConstraints c=new GridBagConstraints();
        c.gridx = x;
        c.gridy=y;
        c.gridwidth=wx;
        c.gridheight = wy;
        l.setConstraints(o,c);
        return o;
    }


    public Component getCustomEditor() {
        JPanel jp = new JPanel();
        GridBagLayout gb=new GridBagLayout();
        jp.setLayout(gb);

        jp.add(setConstr(new JLabel(" width"),gb,0,0,1,1));

        jp.add(setConstr(new RunSlider(RunSlider.HORIZONTAL, 1, 2000, w) {
            public void run() {
                setWidth(getValue());
                getPcs().firePropertyChange("Oval2d", Oval2d.this, null);
            }
        },gb,1,0,3,1));

        jp.add(setConstr(new JLabel(" height"),gb,0,1,1,1));

        jp.add(setConstr(new RunSlider(RunSlider.HORIZONTAL, 1, 2000, h) {
            public void run() {
                setHeight(getValue());
                getPcs().firePropertyChange("Oval2d", Oval2d.this, null);
            }
        },gb,1,1,3,1));

        jp.add(setConstr(new RunButton("setForeground") {
            public void run() {
                Oval2d.this.setForeground(In.getColor());
                getPcs().firePropertyChange("size", Oval2d.this, null);
            }
        },gb,1,3,3,1));

        jp.add(setConstr(getRunAffinePanel(),gb,4,0,GridBagConstraints.REMAINDER,GridBagConstraints.REMAINDER));

        return jp;
    }

    public boolean contains(double x, double y) {
        Shape s = getShape();
        return s.contains(x,y);
    }

    public void move(double x, double y) {
        xc=(int)x;
        yc=(int)y;
        y1=yc-h/2;
        x1=xc-w/2;
    }

    public Shape getShape() {
        Point pc = getCenter();
        Ellipse2D e2d = new Ellipse2D.Double(pc.x - w / 2, pc.y - h / 2, w, h);
        AffineTransform at = getAffineTransform();
        return at.createTransformedShape(e2d);
    }

}
