package j2d;

import j2d.MinMax2D;
import j2d.ShortImageBean;
import j2d.border.BorderUtils;
import utils.ResourceManager;

import java.awt.*;

/**
 * Created by IntelliJ IDEA.
 * User: Cool Ed
 * Date: Mar 9, 2006
 * Time: 3:23:55 AM
 * To change this template use File | Settings | File Templates.
 */
public class SpatialCorrelation {

    public static void main(String[] args) {
          spatialCorrelationTest();
    }

    public static void binaryTemplateTest() {
        ResourceManager rm = ResourceManager.getResourceManager();
        Image template = ImageUtils.getImage(rm.getImageFile("baboon_eye.gif"));
        ImageUtils.displayImage(template,"baboon_eye.gif");
        Image binaryImage = ImageUtils.getBinaryTemplate(template);
        ImageUtils.displayImage(binaryImage,"binaryImage");
    }

    private static void spatialCorrelationTest() {
        ResourceManager rm = ResourceManager.getResourceManager();
        Image template = ImageUtils.getImage(rm.getImageFile("baboon_eye.gif"));

        Image scene = ImageUtils.getImage(rm.getImageFile("baboon.jpg"));
        double varianceOfTheSceneTimesN = ImageUtils.getVarianceTimesN(scene);
        double varianceOfTheTemplateTimesN = ImageUtils.getVarianceTimesN(template);
        double demoninator = Math.sqrt(varianceOfTheSceneTimesN*varianceOfTheTemplateTimesN);
        
        ShortImageBean sceneSquaredSib = new ShortImageBean(scene);
        sceneSquaredSib.square();
        Image sceneSquared = sceneSquaredSib.getImage();
        ImageUtils.displayImage(sceneSquared,"sceneSquaredSib") ;

        ShortImageBean uccfSib = new ShortImageBean(getSpatialCorrelation(template, scene));
        uccfSib.square();
        Image binaryTemplate = ImageUtils.getBinaryTemplate(template);
        ImageUtils.displayImage(binaryTemplate,"binaryTemplate");

        Image energy = getSpatialCorrelation(binaryTemplate, sceneSquared);
        uccfSib.divide(energy);
        ImageUtils.displayImage(uccfSib.getImage(), "NccfImage squared");

    }

    /**
     * Correlate imag1 with img2 and return the new image.
     * This is very slow correlation, but it works well.
     *
     * @param templateImage
     * @param sceneImage
     * @return correlation output
     */
    public static Image getSpatialCorrelation(Image templateImage, Image sceneImage) {
        // Find min and max of template
        ShortImageBean templateSib = new ShortImageBean(templateImage);
        ShortImageBean grownSceneImgBean = new ShortImageBean(sceneImage);

        j2d.MinMax2D mmTemplate = templateSib.getMinMax2DForBorder();

        FloatImageBean templateFloatImgBean = getTemplateFloatImageBean(mmTemplate, templateSib);

        // Now, remove template average
        templateFloatImgBean.removeTheDcComponent();

        // Create blank FloatImageBean of same size as scene image
        j2d.FloatImageBean outputFloatImgBean = new j2d.FloatImageBean(grownSceneImgBean.getWidth(),
                grownSceneImgBean.getHeight());

        //measure template image dimensions
        int tmpltHght = mmTemplate.getMaxY() - mmTemplate.getMinY() + 1;
        int tmpltWid = mmTemplate.getMaxX() - mmTemplate.getMinX() + 1;

        // Create New Grown Image; Grow Image by template size
        int left = tmpltWid / 2;
        int top = tmpltHght / 2;
        int bottom = tmpltHght / 2;
        int right = tmpltWid / 2;

        grownSceneImgBean = new ShortImageBean(
                BorderUtils.getBorderImage(sceneImage,
                        left, right, top, bottom,
                        BorderUtils.BORDER_REFLECT));

        //now perform the correlation between the template and the grown scene image
        double rMax = correlate(templateFloatImgBean.getR(),
                grownSceneImgBean.getR(),
                outputFloatImgBean.getR());
        double gMax = correlate(templateFloatImgBean.getG(),
                grownSceneImgBean.getG(),
                outputFloatImgBean.getG());
        double bMax = correlate(templateFloatImgBean.getB(),
                grownSceneImgBean.getB(),
                outputFloatImgBean.getB());

        // Apply Simple scale to output
        float scale = 255 / (float) Math.max(rMax, Math.max(gMax, bMax));
        outputFloatImgBean.scale(scale);
        return outputFloatImgBean.getImage();
    }

    public static double correlate(float[][] tmpltArray,
                                   short[][] sceneArray,
                                   float[][] outputArray) {
        int sceneW = sceneArray.length;//width is 1st dimension
        int sceneH = sceneArray[0].length;//height is 2nd dimension
        int tmpltW = tmpltArray.length;//width is 1st dimension
        int tmpltH = tmpltArray[0].length;//height is 2nd dimension

        double rSum;
        // other necessary constants
        double rMax = 0;
        for (int x = 0; x < sceneW - tmpltW; x++) {
            for (int y = 0; y < sceneH - tmpltH; y++) {
                rSum = 0;
                for (int tx = 0; tx < tmpltW; tx++) {
                    for (int ty = 0; ty < tmpltH; ty++) {
                        rSum += tmpltArray[tx][ty] * sceneArray[tx + x][ty + y];
                    }
                }
                rSum /= tmpltW * tmpltH;
                rMax = Math.max(rMax, rSum);
                outputArray[x][y] = (float) (Math.max(0, rSum));
            }
        }
        return rMax;
    }

    private static j2d.FloatImageBean getTemplateFloatImageBean(MinMax2D mmTemplate,
                                                            ShortImageBean templateSib) {
        return new j2d.FloatImageBean(
                getTemplate(mmTemplate, templateSib.getR()),
                getTemplate(mmTemplate, templateSib.getG()),
                getTemplate(mmTemplate, templateSib.getB()));
    }

    private static float[][] getTemplate(MinMax2D mm, short[][] r) {
        Dimension mmSize = mm.getSize();
        float[][] rt = new float[mmSize.width][mmSize.height];
        for (int x = 0; x < mmSize.width; x++) {
            for (int y = 0; y < mmSize.height; y++) {
                rt[x][y] = r[x + mm.getMinX()][y + mm.getMinY()];
            }
        }
        return rt;
    }
}
