/*
 * Copyright (C) 1996 Emanuel Borsboom <manny@zerius.victoria.bc.ca>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (affineTransform your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sound.zrs.synthgen;

import java.util.Enumeration;
import java.util.Vector;

abstract class Generators extends GeneratorModel {
    private Vector generatorVector = new Vector();

    public void init() {
        generatorVector = new Vector();
    }

    public Generators(Object o) {
        super(null);
    }

    public void recalculate() {
        Enumeration e = generatorVector.elements();
        while (e.hasMoreElements())
            ((GeneratorModel) e.nextElement()).recalculate();
    }

    public GeneratorModel add(GeneratorModel gen) {
        generatorVector.addElement(gen);
        return gen;
    }
}

public class SynthesizerGeneratorModel extends Generators {

    int samplingRate;
    int sampleNo;
    double currentTime;
    double timeIncrement;

    public double maxValue;

    GeneratorModel outputGeneratorModel;

    public SynthesizerGeneratorModel(int samplingRate) {
        super(null);
        init(samplingRate);
    }

    public SynthesizerGeneratorModel(SynthesizerGeneratorModel parent) {
        super(parent);
        init(parent.samplingRate);
    }

    private void init(int samplingRate) {
        this.samplingRate = samplingRate;
        timeIncrement = 1 / (double) samplingRate;
        maxValue = 0.0;
        sampleNo = -1;
        super.init();
    }

    public void setOutput(GeneratorModel outputGeneratorModel) {
        this.outputGeneratorModel = outputGeneratorModel;
    }

    public double nextValue() throws SynthesizerException {
        sampleNo++;
        currentTime = (double) sampleNo * timeIncrement;
        super.recalculate();


        double v = outputGeneratorModel.getValue();

        if (Math.abs(v) > maxValue)
            maxValue = Math.abs(v);

        return v;
    }

}
